# Extract  PCLinux name and version
%define distro_version	%(perl -ne '/^([.\\w\\s]+) \\(.+\\).+/ and print $1' < /etc/release)

# To bootstrap for example py3.x -> py3.y upgrades
%bcond_without python

%global _python_bytecompile_extra 0

%{?scl:%scl_package gdb}
%{!?scl:
 %global pkg_name %{name}
 %global _root_prefix %{_prefix}
 %global _root_datadir %{_datadir}
 %global _root_libdir %{_libdir}
}

%global have_libipt 1
Name: %{?scl_prefix}gdb

# Freeze it when GDB gets branched
%global snapsrc    20220501
# See timestamp of source gnulib installed into gnulib/ .
%global snapgnulib 20210105
%global tarname gdb-%{version}
Version: 13.1

# The release always contains a leading reserved number, start it at 1.
# `upstream' is not a part of `name' to stay fully rpm dependencies compatible for the testing.
Release: %mkrel 1
License: GPLv3+ and GPLv3+ with exceptions and GPLv2+ and GPLv2+ with exceptions and GPL+ and LGPLv2+ and LGPLv3+ and BSD and Public Domain and GFDL
Group:   Development/Other
Source: ftp://sourceware.org/pub/gdb/releases/%{tarname}.tar.xz
URL: https://gnu.org/software/gdb/

# For our convenience
%global gdb_src %{tarname}
%global gdb_build build-%{_target_platform}

Summary: A stub package for GNU source-level debugger

%global have_inproctrace 1

Recommends: gcc-plugins

%global librpmver 1
%if 0%{?__isa_bits} == 64
%global librpmname librpm.so.%{librpmver}()(64bit)
%else
%global librpmname librpm.so.%{librpmver}
%endif

Source2: gdb-orphanripper.c
Source3: gdb-gstack.man

# /etc/gdbinit (from Debian but with Fedora compliant location).
#=fedora
Source4: gdbinit
# Provide gdbtui as it is removed upstream (BZ 797664).
Source5: gdbtui

# Fedora patches:
Patch9998: _gdb.spec.Patch.include
Patch9999: _gdb.spec.patch.include
%include %{PATCH9998}

# Mageia patches:
Patch10001: fix-build-with-autoconf-2.69+.patch
Patch10002: gdb-autoconf.patch

# PCLinux patches:
#note that we are using a custom gdb-6.6-buildid-locate-rpm.patch so..
#take care of above when update this package
Patch11001: gdb-13.1-find-python.patch

BuildRequires: autoconf
BuildRequires:	rust
BuildRequires:	flex
BuildRequires:	bison
BuildRequires:	%{_lib}ipt-devel
BuildRequires:	%{_lib}mpfr-devel
BuildRequires:	%{_lib}ncurses-devel
BuildRequires:	%{_lib}expat-devel
BuildRequires:	%{_lib}python3-devel
BuildRequires:	%{_lib}rpm-devel
BuildRequires:	%{_lib}readline-devel
BuildRequires:	%{_lib}babeltrace-devel
BuildRequires:	%{_lib}xxhash-devel
BuildRequires:	%{_lib}guile-devel
BuildRequires:	texinfo

%description
Gdb is a full featured, command driven debugger. Gdb allows you to
trace the execution of programs and examine their internal state at
any time.  Gdb works for C and C++ compiled with the GNU C compiler
gcc.

If you are going to develop C and/or C++ programs and use the GNU gcc
compiler, you may want to install gdb to help you debug your programs.

%package gdbserver
Summary: A standalone server for GDB (the GNU source-level debugger)
Group:   Development/Other
 
%description gdbserver
GDB, the GNU debugger, allows you to debug programs written in C, C++,
Java, and other languages, by executing them in a controlled fashion
and printing their data.

This package provides a program that allows you to run GDB on a different
machine than the one which is running the program being debugged.

%package doc
Summary: Documentation for GDB (the GNU source-level debugger)
License: GFDL
Group: Documentation
BuildArch: noarch

%description doc
GDB, the GNU debugger, allows you to debug programs written in C, C++,
Java, and other languages, by executing them in a controlled fashion
and printing their data.

This package provides INFO user manual for GDB.

%prep
%setup -q -n %{gdb_src}

# Files have `# <number> <file>' statements breaking VPATH / find-debuginfo.sh .
(cd gdb;rm -fv $(perl -pe 's/\\\n/ /' <Makefile.in|sed -n 's/^YYFILES = //p'))

# *.info* is needlessly split in the distro tar; also it would not get used as
# we build in %{gdb_build}, just to be sure.
find -name "*.info*"|xargs rm -f

%include %{PATCH9999}

%patch10001 -p1
%patch10002 -p1
%patch11001 -p1

pushd libiberty
autoconf -f
popd
pushd intl
autoconf -f
popd

find -name "*.orig" | xargs rm -f
! find -name "*.rej" # Should not happen.

cat > gdb/version.in << _FOO
%{version}-%{release} (%{distro_version})
_FOO

rm -f libdecnumber/gstdint.h
rm -f bfd/doc/*.info
rm -f bfd/doc/*.info-*
rm -f gdb/doc/*.info
rm -f gdb/doc/*.info-*

mv -f readline/readline/doc readline-doc
rm -rf readline/readline/*
mv -f readline-doc readline/readline/doc
rm -rf zlib texinfo

%build

test -e %{_root_libdir}/librpm.so.%{librpmver}

# A set of common GDB configure flags
COMMON_GDB_CONFIGURE_FLAGS="\
	--prefix=%{_prefix}					\
	--libdir=%{_libdir}					\
	--sysconfdir=%{_sysconfdir}				\
	--mandir=%{_mandir}					\
	--infodir=%{_infodir}					\
	--with-system-gdbinit=%{_sysconfdir}/gdbinit		\
	--with-gdb-datadir=%{_datadir}/gdb			\
	--enable-gdb-build-warnings=,-Wno-unused		\
	--enable-build-with-cxx					\
	--disable-werror						\
	--with-separate-debug-dir=/usr/lib/debug		\
	--disable-sim						\
	--disable-rpath						\
	--without-stage1-ldflags				\
	--disable-libmcheck					\
	--with-system-readline					\
	--without-libunwind					\
	--enable-64-bit-bfd					\
	--with-mpfr						\
	--with-system-zlib					\
	--with-lzma						\
    --with-debuginfod					\
"

for fprofile in %{?_with_profile:-fprofile} ""
do

# Now we build the GDB.
mkdir %{gdb_build}$fprofile
cd %{gdb_build}$fprofile

export CFLAGS="$RPM_OPT_FLAGS %{?_with_asan:-fsanitize=address}"
export LDFLAGS="%{?__global_ldflags} %{?_with_asan:-fsanitize=address}"

CFLAGS="$CFLAGS -DNEED_RL_STATE_FEDORA_GDB"

# The configure flags we will use when building the full GDB.
GDB_FULL_CONFIGURE_FLAGS="\
	--with-babeltrace					\
	--with-expat						\
$(: ppc64 host build crashes on ppc variant of libexpat.so )	\
	--without-libexpat-prefix				\
	--enable-tui						\
	--with-python				\
	--with-guile				\
%if %{have_inproctrace}
	--enable-inprocess-agent				\
%else
	--disable-inprocess-agent				\
%endif
%if %{have_libipt}
	--with-intel-pt						\
%else
	--without-intel-pt					\
%endif
	--with-xxhash						\
	--with-rpm=librpm.so.%{librpmver}			\
	--enable-unit-tests"

../configure							\
	${COMMON_GDB_CONFIGURE_FLAGS}				\
	${GDB_FULL_CONFIGURE_FLAGS}				\
	--with-auto-load-dir='$debugdir:$datadir/auto-load%{?scl::%{_root_datadir}/gdb/auto-load}'	\
	--with-auto-load-safe-path='$debugdir:$datadir/auto-load%{?scl::%{_root_datadir}/gdb/auto-load}'	\
	--enable-targets=arm-linux-gnu,aarch64-linux-gnu	\
	%{_target_platform}

if [ -z "%{!?_with_profile:no}" ]
then
  # Run all the configure tests being incompatible with $FPROFILE_CFLAGS.
%make_build configure-host configure-target
%make_build clean
%make_build -C gdb/gdbserver linux-x86-low.o
fi

if [ "$fprofile" = "-fprofile" ]
then
  FPROFILE_CFLAGS='-fprofile-generate'
elif [ -z "%{!?_with_profile:no}" ]
then
  FPROFILE_CFLAGS='-fprofile-use'
  (cd ../${builddir}-fprofile;
   # It was 333 on x86_64.
   test $(find -name "*.gcda"|wc -l) -gt 300
   find -name "*.gcda" | while read -r i
   do
     ln $i ../${builddir}/$i
   done
  )
else
  FPROFILE_CFLAGS=""
fi

# Prepare gdb/config.h first.
%make_build CFLAGS="$CFLAGS $FPROFILE_CFLAGS" LDFLAGS="$LDFLAGS $FPROFILE_CFLAGS" V=1 maybe-configure-gdb
perl -i.relocatable -pe 's/^(D\[".*_RELOCATABLE"\]=" )1(")$/${1}0$2/' gdb/config.status

%make_build CFLAGS="$CFLAGS $FPROFILE_CFLAGS" LDFLAGS="$LDFLAGS $FPROFILE_CFLAGS" V=1

! grep '_RELOCATABLE.*1' gdb/config.h

if [ "$fprofile" = "-fprofile" ]
then
  cd gdb
  cp -p gdb gdb-withindex
  PATH="$PWD:$PATH" sh ../../gdb/gdb-add-index $PWD/gdb-withindex
  ./gdb -nx -ex q ./gdb-withindex
  ./gdb -nx -readnow -ex q ./gdb-withindex
  cd ..
fi

cd ..

done	# fprofile

cd %{gdb_build}

# Copy the <sourcetree>/gdb/NEWS file to the directory above it.
cp $RPM_BUILD_DIR/%{gdb_src}/gdb/NEWS $RPM_BUILD_DIR/%{gdb_src}

%check
cd %{gdb_build}

# We always run the unittests.
(cd gdb; make run GDBFLAGS='-batch -ex "maintenance selftest"')

%if 0%{!?_with_testsuite:1}
echo ====================TESTSUITE DISABLED=========================
%else
echo ====================TESTING=========================
cd gdb
gcc -o ./orphanripper %{SOURCE2} -Wall -lutil -ggdb2
(
  ulimit -H -c
  ulimit -c unlimited || :
  echo 'int main (void) { return 0; }' >biarch.c
  CHECK=""
  for BI in -m64 -m32 -m31 ""
  do
    if [ -z "$BI" -a -n "$CHECK" ];then
      continue
    fi
    if ! gcc $BI -o biarch biarch.c
    then
      continue
    fi
    CHECK="$CHECK check//unix/$BI check//native-gdbserver/$BI check//native-extended-gdbserver/$BI"
  done
  gcc $RPM_OPT_FLAGS -o biarch biarch.c
  RPM_SIZE="$(file ./biarch|sed -n 's/^.*: ELF \(32\|64\)-bit .*$/\1/p')"
  if [ "$RPM_SIZE" != "64" ]
  then
    CHECK="$(echo " $CHECK "|sed 's#check//unix/-m64 check//native-gdbserver/-m64 check//native-extended-gdbserver/-m64# #')"
  fi

  for test in				\
    gdb.base/readline-overflow.exp	\
    gdb.base/bigcore.exp		\
  ; do
    mv -f ../../gdb/testsuite/$test ../gdb/testsuite/$test-DISABLED || :
  done

  ./orphanripper %make_build -k $CHECK || :
)
for t in sum log
do
  for file in testsuite*/gdb.$t
  do
    suffix="${file#testsuite}"
    suffix="${suffix%/gdb.$t}"
    ln $file gdb-%{_target_platform}$suffix.$t || :
  done
done
tar cjf gdb-%{_target_platform}.tar.bz2 gdb-%{_target_platform}*.{sum,log}
uuencode gdb-%{_target_platform}.tar.bz2 gdb-%{_target_platform}.tar.bz2
cd ../..
echo ====================TESTING END=====================
%endif

%install
rm -rf $RPM_BUILD_ROOT
cd %{gdb_build}
%make_install

mkdir -p $RPM_BUILD_ROOT%{_prefix}/libexec
mv -f $RPM_BUILD_ROOT%{_bindir}/gdb $RPM_BUILD_ROOT%{_prefix}/libexec/gdb
ln -s -r                                                 $RPM_BUILD_ROOT%{_prefix}/libexec/gdb  $RPM_BUILD_ROOT%{_bindir}/gdb

mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/gdbinit.d
touch -r %{SOURCE4} $RPM_BUILD_ROOT%{_sysconfdir}/gdbinit.d
sed 's#%%{_sysconfdir}#%{_sysconfdir}#g' <%{SOURCE4} >$RPM_BUILD_ROOT%{_sysconfdir}/gdbinit
touch -r %{SOURCE4} $RPM_BUILD_ROOT%{_sysconfdir}/gdbinit

install -m 755 %{SOURCE5} $RPM_BUILD_ROOT%{_bindir}/gdbtui
ln -sf gdb.1 $RPM_BUILD_ROOT%{_mandir}/man1/gdbtui.1

for i in `find $RPM_BUILD_ROOT%{_datadir}/gdb/python/gdb -name "*.py"`
do
  # Files could be also patched getting the current time.
  touch -r $RPM_BUILD_DIR/%{gdb_src}/gdb/version.in $i
done

# we don't want these
find %{buildroot} -name "*.la" -delete
find %{buildroot} -name "*.a" -delete

# Compile python files
%py_byte_compile %{__python3} %{buildroot}%{_datadir}/gdb/python/gdb

rm -rf $RPM_BUILD_ROOT%{_datadir}/locale/
rm -f $RPM_BUILD_ROOT%{_infodir}/bfd*
rm -f $RPM_BUILD_ROOT%{_infodir}/standard*
rm -f $RPM_BUILD_ROOT%{_infodir}/configure*

rm -rf $RPM_BUILD_ROOT%{_includedir}/*.h
rm -rf $RPM_BUILD_ROOT/%{_libdir}/lib{bfd*,opcodes*,iberty*,ctf*}

cp -p %{SOURCE3} $RPM_BUILD_ROOT%{_mandir}/man1/gstack.1
ln -s gstack.1 $RPM_BUILD_ROOT%{_mandir}/man1/pstack.1
ln -s gstack $RPM_BUILD_ROOT%{_bindir}/pstack

(cd $RPM_BUILD_ROOT%{_datadir}/gdb/syscalls
 rm -f mips*.xml
%ifnarch sparc sparcv9 sparc64
 rm -f sparc*.xml
%endif
%ifnarch x86_64
 rm -f amd64-linux.xml
%endif
%ifnarch %{ix86} x86_64
 rm -f i386-linux.xml
%endif
)

rm -f $RPM_BUILD_ROOT%{_infodir}/gdbint*
rm -f $RPM_BUILD_ROOT%{_infodir}/stabs*
rm -f $RPM_BUILD_ROOT%{_infodir}/ctf-spec*
rm -f $RPM_BUILD_ROOT%{_infodir}/dir
rm -f $RPM_BUILD_ROOT%{_datadir}/gdb/system-gdbinit/elinos.py
rm -f $RPM_BUILD_ROOT%{_datadir}/gdb/system-gdbinit/wrs-linux.py
rmdir $RPM_BUILD_ROOT%{_datadir}/gdb/system-gdbinit

%files
%doc README NEWS COPYING3 COPYING COPYING.LIB
%{_bindir}/gdb
%{_bindir}/gdbtui
%{_bindir}/gcore
%{_mandir}/*/gcore.1*
%{_bindir}/gstack
%{_mandir}/*/gstack.1*
%{_bindir}/pstack
%{_mandir}/*/pstack.1*
%{_mandir}/*/gdbtui.1*
%{_includedir}/gdb

%{_prefix}/libexec/gdb
%config(noreplace) %{_sysconfdir}/gdbinit
%{_mandir}/*/gdb.1*
%{_sysconfdir}/gdbinit.d
%{_mandir}/*/gdbinit.5*
%{_bindir}/gdb-add-index
%{_mandir}/*/gdb-add-index.1*
%{_datadir}/gdb

%files gdbserver
%{_bindir}/gdbserver
%{_mandir}/*/gdbserver.1*
%if %{have_inproctrace}
%{_libdir}/libinproctrace.so
%endif

%files doc
%{_infodir}/annotate.info*
%{_infodir}/gdb.info*
%{_infodir}/sframe-spec.info*


%changelog
* Mon Mar 13 2023 yodelu 13.1-1pclos2023
- version update

* Mon Feb 13 2023 yodelu 11.2-1pclos2023
- version update

* Wed Oct 27 2021 yodelu 11.1-1pclos2021
- update

* Wed Aug 20 2014 ghostbunny <ghostbunny at gmx dot de> 7.8-1pclos2014
- 7.8
- recompress source to tar.xz
- set separate debug dir to _libdir instead of prefix/lib
- fix files section for new files

* Tue Aug 20 2013 billybot <billybot> 7.6-1pclos2013
- update python 2.7

* Sun Sep 12 2010 Texstar <texstar at gmail.com> 7.1-2pclos2010
- rebuild
